import multer from 'multer';
import { v4 as uuidv4 } from 'uuid';
import { Product } from '../service/index.js';
import { Request } from '../util/index.js';
const storage = multer.diskStorage({
    destination: (req, file, cb) => {
        cb(null, process.env.IMAGES_PATH);
    },
    filename: (req, file, cb) => {
        const mimetypeMap = {
            'image/png': 'png',
            'image/jpeg': 'jpeg',
        };
        const extension = (mimetypeMap)[file.mimetype] || null;
        if (!extension) {
            req.params.productImage = 'INVALID_IMAGE';
            return;
        }
        const fileName = `${uuidv4()}.${extension}`;
        req.params.productImage = fileName;
        cb(null, fileName);
    },
});
const upload = multer({ storage });
class ProductRoutes {
    register(app) {
        app.post('/product', upload.single('file'), async (req, res) => {
            try {
                const data = JSON.parse(req.body.data);
                const response = await Product.create({ ...Request.getAllParams(req), ...data });
                res.status(201).send(response);
            }
            catch (err) {
                Request.handleResponseError(res, err);
            }
        });
        app.get('/product', async (req, res) => {
            try {
                const response = await Product.list();
                res.send(response);
            }
            catch (err) {
                Request.handleResponseError(res, err);
            }
        });
        app.put('/product', upload.single('file'), async (req, res) => {
            try {
                const data = JSON.parse(req.body.data);
                const response = await Product.modify({ ...data, productImage: req.params.productImage });
                const status = ('circularDependency' in (response || {})) ? 500 : 200;
                res.status(status).send(response);
            }
            catch (err) {
                Request.handleResponseError(res, err);
            }
        });
        app.delete('/product', async (req, res) => {
            try {
                const response = await Product.remove(Request.getAllParams(req));
                if ('error' in response) {
                    res.status(500).send(response);
                }
                else {
                    res.send(response);
                }
            }
            catch (err) {
                Request.handleResponseError(res, err);
            }
        });
    }
}
const instance = new ProductRoutes();
export { instance as ProductRoutes };
export default instance;
